@@ -47,6 +47,7 @@ Clock::Clock(QObject *parent, const QVariantList &args)
       m_layout(0)
 {
     KGlobal::locale()->insertCatalog("libplasmaclock");
+    KGlobal::locale()->insertCatalog("timezones4");
     setHasConfigurationInterface(true);
     setBackgroundHints(Plasma::Applet::DefaultBackground);
     //If we do not set this, the user has to press CTRL when shrinking the plasmoid on the desktop beyond a certain size.
@@ -313,7 +314,9 @@ void Clock::calculateDateString()
 //     }
 
     if( m_showTimezone == true ) {
-        m_timezoneString = '(' + currentTimezone() + ')';
+        QString timezonetranslated = i18n( currentTimezone().toUtf8().data());
+        timezonetranslated = timezonetranslated.replace('_', ' ');
+        m_timezoneString = '(' + timezonetranslated  + ')';
     }
 }
 
@@ -533,7 +536,11 @@ if ( contentsRect().size().width() > m_timeStringSize.width() && (formFactor() =
         //resize ( QSizeF ( m_contentSize.width(),geometry().size().height() ) );
     } else {
         //add margins
-        resize ( m_contentSize + QSizeF(size()-contentsRect().size()) );
+        resize(m_contentSize + QSizeF(size()-contentsRect().size()));
+        setPreferredSize(m_contentSize + QSizeF(size()-contentsRect().size()));
+        resize(preferredSize());
+        emit sizeHintChanged(Qt::PreferredSize);
+        emit appletTransformedItself();
     }
 
 } else { //in a panel or timestring wider than plasmoid -> change size to the minimal needed space, i.e. the timestring will not increase in point-size OR plasmoid in Panel.
@@ -666,7 +673,10 @@ if ( contentsRect().size().width() > m_timeStringSize.width() && (formFactor() =
 
         //we use the minimal height here, since the user has given us too much height we cannot use for anything useful. minimal width because we are in a panel.
         kDebug() << "we set the minimum size needed as the size we want";
-        resize ( QSizeF ( m_minimumContentSize.width() + m_margin*2,m_minimumContentSize.height() ) + (size() - contentsRect().size()) );
+        setPreferredSize ( QSizeF ( m_minimumContentSize.width() + m_margin*2,m_minimumContentSize.height() ) + (size() - contentsRect().size()) );
+        resize(preferredSize());
+        emit sizeHintChanged(Qt::PreferredSize);
+        emit appletTransformedItself();
     }
 }
 }
