@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
+ *   Copyright (C) 2007 Ivan Cukic <ivan.cukic(at)kde.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License version 2,
@@ -17,8 +17,8 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-#ifndef LANCELOTAPP_MODELS_CONTACTSKOPETE_H_
-#define LANCELOTAPP_MODELS_CONTACTSKOPETE_H_
+#ifndef LANCELOTAPP_MODELS_CONTACTSKOPETE_H
+#define LANCELOTAPP_MODELS_CONTACTSKOPETE_H
 
 #include "kopete_interface.h"
 #include "BaseModel.h"
@@ -31,21 +31,28 @@ public:
     ContactsKopete();
     ~ContactsKopete();
 
-    void timerEvent(QTimerEvent * event);
-
 protected:
     void activate(int index);
     void load();
+    void load(bool forceReload);
+    void updateContactData(const QString & contactId);
+    void timerEvent(QTimerEvent * event);
+
+protected Q_SLOTS:
+    void contactChanged(const QString & contactId);
 
 private:
     org::kde::Kopete * m_interface;
-    QBasicTimer m_timer;
+    QBasicTimer m_delayTimer;
+    QBasicTimer m_checkRunningTimer;
+    QStringList m_contactsToUpdate;
     QString m_kopeteAvatarsDir;
-    bool m_kopeteRunning;
+    bool m_kopeteRunning : 1;
+    bool m_noOnlineContacts : 1;
 };
 
 } // namespace Models
 
-#endif // LANCELOTAPP_MODELS_CONTACTSKOPETE_H_
+#endif // LANCELOTAPP_MODELS_CONTACTSKOPETE_H
 
 
