@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
+ *   Copyright (C) 2007 Ivan Cukic <ivan.cukic(at)kde.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License version 2,
@@ -17,24 +17,23 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-#ifndef LANCELOTAPP_MODELS_NEWDOCUMENTS_H_
-#define LANCELOTAPP_MODELS_NEWDOCUMENTS_H_
+#ifndef LANCELOTAPP_MODELS_NEWDOCUMENTS_H
+#define LANCELOTAPP_MODELS_NEWDOCUMENTS_H
 
-#include "BaseModel.h"
+#include "FolderModel.h"
 
 namespace Models {
 
-class NewDocuments : public BaseModel {
+class NewDocuments : public FolderModel {
     Q_OBJECT
 public:
     NewDocuments();
     virtual ~NewDocuments();
 
-protected:
-    void load();
+    static QString path();
 
 };
 
 } // namespace Models
 
-#endif /* LANCELOTAPP_MODELS_NEWDOCUMENTS_H_ */
+#endif /* LANCELOTAPP_MODELS_NEWDOCUMENTS_H */
