@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
+ *   Copyright (C) 2007 Ivan Cukic <ivan.cukic(at)kde.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Lesser/Library General Public License version 2,
@@ -18,29 +18,23 @@
  */
 
 #include "BasicWidget.h"
-#include <KDebug>
-#include <cmath>
 #include "Global.h"
+
+#include <QApplication>
+#include <QtGui/QPainter>
+
+#include <KGlobalSettings>
+
+#include <plasma/paintutils.h>
+
+#include <cmath>
+
 #include <lancelot/lancelot.h>
 
 #define WIDGET_PADDING 8
 
 #define max(A, B) ((A) >= (B)) ? (A) : (B)
 
-// macro for setting the left coordinate of items
-// relative to the parent and with alignment
-// taken into consideration
-#define setLeft(itemRect, parentRect, alignment) \
-    if ((parentRect).width() > (itemRect).width()) { \
-        if ((alignment) & Qt::AlignHCenter) \
-        (itemRect).moveLeft(WIDGET_PADDING + ((parentRect).width() - (itemRect).width()) / 2); \
-        else if ((alignment) & Qt::AlignRight) \
-        (itemRect).moveLeft(WIDGET_PADDING + (parentRect).width() - (itemRect).width()); \
-    } else { \
-        (itemRect).setWidth((parentRect).width()); \
-        (itemRect).moveLeft(WIDGET_PADDING); \
-    };
-
 namespace Lancelot
 {
 
@@ -74,10 +68,30 @@ class BasicWidget::Private {
     void init(BasicWidget * parent)
     {
         parent->setAcceptsHoverEvents(true);
-        parent->resize(140, 38);
+        // parent->resize(140, 38);
         parent->setGroupByName("BasicWidget");
     }
 
+    int shortcutPosition(QString & text)
+    {
+        Q_UNUSED(text);
+
+        int index = 0;
+        while ((index = text.indexOf('&', index)) != -1) {
+            if (index == text.size() - 1) {
+                return -1;
+            }
+
+            if (text.at(index + 1) != '&') {
+                return index + 1;
+            }
+
+            index++;
+            text.remove(index, 1);
+        }
+        return -1;
+    }
+
     QIcon icon;
     Plasma::Svg iconInSvg;
     QSize iconSize;
@@ -90,39 +104,38 @@ class BasicWidget::Private {
 };
 
 BasicWidget::BasicWidget(QGraphicsItem * parent)
-  : Widget(parent),
+  : Widget(),
     d(new Private(this))
 {
-    L_WIDGET_SET_INITIALIZED;
+    setParentItem(parent);
 }
 
 BasicWidget::BasicWidget(QString title, QString description,
         QGraphicsItem * parent)
-  : Widget(parent),
+  : Widget(),
     d(new Private(this, title, description))
 {
-    L_WIDGET_SET_INITIALIZED;
+    setParentItem(parent);
 }
 
 BasicWidget::BasicWidget(QIcon icon, QString title,
         QString description, QGraphicsItem * parent)
-  : Widget(parent),
+  : Widget(),
     d(new Private(this, icon, title, description))
 {
-    L_WIDGET_SET_INITIALIZED;
+    setParentItem(parent);
 }
 
 BasicWidget::BasicWidget(const Plasma::Svg & icon, QString title,
         QString description, QGraphicsItem * parent)
-  : Widget(parent),
+  : Widget(),
     d(new Private(this, icon, title, description))
 {
-    L_WIDGET_SET_INITIALIZED;
+    setParentItem(parent);
 }
 
 BasicWidget::~BasicWidget()
 {
-    L_WIDGET_UNSET_INITIALIZED;
     delete d;
 }
 
@@ -136,9 +149,23 @@ void BasicWidget::paint(QPainter * painter,
     paintForeground(painter);
 }
 
+// macro for setting the left coordinate of items
+// relative to the parent and with alignment
+// taken into consideration
+#define setLeft(itemRect, parentRect, alignment) \
+    if ((parentRect).width() > (itemRect).width()) { \
+        if ((alignment) & Qt::AlignHCenter) \
+        (itemRect).moveLeft(WIDGET_PADDING + ((parentRect).width() - (itemRect).width()) / 2); \
+        else if ((alignment) & Qt::AlignRight) \
+        (itemRect).moveLeft(WIDGET_PADDING + (parentRect).width() - (itemRect).width()); \
+    } else { \
+        (itemRect).setWidth((parentRect).width()); \
+        (itemRect).moveLeft(WIDGET_PADDING); \
+    };
+
 void BasicWidget::paintForeground(QPainter * painter)
 {
-    if (!L_WIDGET_IS_INITIALIZED) return;
+    bool rtl = QApplication::isRightToLeft();
 
     QPainter * _painter = painter;
 
@@ -149,22 +176,31 @@ void BasicWidget::paintForeground(QPainter * painter)
     QPainter fpainter(&foreground);
     painter = &fpainter;
 
+    QColor fgColor;
     if (!isEnabled()) {
-        painter->setPen(QPen(group()->foregroundColor()->disabled));
+        fgColor = group()->foregroundColor()->disabled;
     } else if (isHovered()) {
-        painter->setPen(QPen(group()->foregroundColor()->active));
+        fgColor = group()->foregroundColor()->active;
     } else {
-        painter->setPen(QPen(group()->foregroundColor()->normal));
+        fgColor = group()->foregroundColor()->normal;
     }
+    painter->setPen(QPen(fgColor));
 
     QFont titleFont = painter->font();
-    QFont descriptionFont = painter->font();
-    descriptionFont.setPointSize(descriptionFont.pointSize() - 2);
+    QFont descriptionFont = KGlobalSettings::smallestReadableFont();
 
-    QRectF widgetRect       = QRectF(0, 0, size().width() - 2 * WIDGET_PADDING, size().height() - 2 * WIDGET_PADDING);
-    QRectF iconRect         = QRectF(0, 0, d->iconSize.width(), d->iconSize.height());
+    QRectF widgetRect     = QRectF(0, 0, size().width() - 2 * WIDGET_PADDING, size().height() - 2 * WIDGET_PADDING);
+    QRectF iconRect;
 
-    if (d->icon.isNull() && !d->iconInSvg.isValid()) iconRect = QRectF(0, 0, 0, 0);
+    if (!d->icon.isNull() || d->iconInSvg.isValid()) {
+        iconRect = QRectF(QPointF(), d->iconSize);
+        if (iconRect.width() > geometry().width()) {
+            iconRect.setWidth(geometry().width());
+        }
+        if (iconRect.height() > geometry().height()) {
+            iconRect.setHeight(geometry().height());
+        }
+    }
 
     // painter->setFont(titleFont)); // NOT NEEDED
     QRectF titleRect        = painter->boundingRect(widgetRect,
@@ -178,6 +214,7 @@ void BasicWidget::paintForeground(QPainter * painter)
 
         // Modified setLeft macro for icon since we can not cut it if it's larger than needed
         // setLeft(iconRect, widgetRect, d->alignment);
+
         if (d->alignment & Qt::AlignHCenter) {
             iconRect.moveLeft(WIDGET_PADDING + (widgetRect.width() - iconRect.width()) / 2);
         } else if (d->alignment & Qt::AlignRight) {
@@ -187,7 +224,7 @@ void BasicWidget::paintForeground(QPainter * painter)
         setLeft(titleRect, widgetRect, d->alignment);
         setLeft(descriptionRect, widgetRect, d->alignment);
 
-        float top = WIDGET_PADDING, height =
+        qreal top = WIDGET_PADDING, height =
             iconRect.height() + titleRect.height() + descriptionRect.height();
 
         if ((!d->icon.isNull() || d->iconInSvg.isValid()) && !(d->title.isEmpty() && d->description.isEmpty()))
@@ -204,7 +241,7 @@ void BasicWidget::paintForeground(QPainter * painter)
             if (!d->icon.isNull()) {
                 d->icon.paint(_painter, rect);
             } else {
-                d->iconInSvg.resize(d->iconSize);
+                d->iconInSvg.resize(rect.size());
                 d->iconInSvg.paint(_painter, rect.left(), rect.top(), isHovered()?"active":"inactive");
             }
             top += d->iconSize.height() + WIDGET_PADDING;
@@ -213,8 +250,10 @@ void BasicWidget::paintForeground(QPainter * painter)
         if (!d->title.isEmpty()) {
             titleRect.moveTop(top);
             painter->setFont(titleFont);
-            painter->drawText(titleRect,
-                    Qt::AlignLeft | Qt::AlignTop | Qt::TextSingleLine | Qt::ElideRight, d->title);
+            // painter->drawText(titleRect,
+            //        Qt::AlignLeft | Qt::AlignTop | Qt::TextSingleLine | Qt::ElideRight, d->title);
+            drawText(painter, titleRect,
+                   Qt::AlignLeft | Qt::AlignTop | Qt::TextSingleLine | Qt::ElideRight, d->title, true);
             top += titleRect.height();
         }
 
@@ -222,11 +261,14 @@ void BasicWidget::paintForeground(QPainter * painter)
             descriptionRect.moveTop(top);
 
             painter->setFont(descriptionFont);
-            painter->drawText(descriptionRect,
-                    Qt::AlignLeft | Qt::AlignTop | Qt::TextSingleLine | Qt::ElideRight, d->description);
+            // painter->drawText(descriptionRect,
+            //         Qt::AlignLeft | Qt::AlignTop | Qt::TextSingleLine | Qt::ElideRight, d->description);
+            drawText(painter, descriptionRect,
+                    Qt::AlignLeft | Qt::AlignTop | Qt::TextSingleLine | Qt::ElideRight, d->description, false);
         }
     } else {
-        float /*left = WIDGET_PADDING,*/ width =
+        // Horizontal layout
+        qreal /*left = WIDGET_PADDING,*/ width =
             iconRect.width() + fmaxf(titleRect.width(), descriptionRect.width()) +
             WIDGET_PADDING;
 
@@ -244,16 +286,36 @@ void BasicWidget::paintForeground(QPainter * painter)
         }
 
         if ((widgetRect.width() < width) || (d->alignment & Qt::AlignLeft)) {
-            iconRect.moveLeft(WIDGET_PADDING);
-            titleRect.setWidth(widgetRect.width() - ((!d->icon.isNull() || d->iconInSvg.isValid()) ? iconRect.width() + WIDGET_PADDING : 0));
-            descriptionRect.setWidth(titleRect.width());
+            if (rtl) {
+                iconRect.moveRight(widgetRect.right() + WIDGET_PADDING);
+                titleRect.setWidth(widgetRect.width() - ((!d->icon.isNull() || d->iconInSvg.isValid()) ? iconRect.width() + WIDGET_PADDING : 0));
+                descriptionRect.setWidth(titleRect.width());
+            } else {
+                iconRect.moveLeft(WIDGET_PADDING);
+                titleRect.setWidth(widgetRect.width() - ((!d->icon.isNull() || d->iconInSvg.isValid()) ? iconRect.width() + WIDGET_PADDING : 0));
+                descriptionRect.setWidth(titleRect.width());
+            }
         } else if (d->alignment & Qt::AlignHCenter) {
-            iconRect.moveLeft(WIDGET_PADDING + (widgetRect.width() - width) / 2);
-        } else
-            iconRect.moveLeft(WIDGET_PADDING + (widgetRect.width() - width));
+            if (rtl) {
+                iconRect.moveRight(WIDGET_PADDING + (widgetRect.width() + width) / 2);
+            } else {
+                iconRect.moveLeft(WIDGET_PADDING + (widgetRect.width() - width) / 2);
+            }
+        } else {
+            if (rtl) {
+                iconRect.moveRight(widgetRect.width() - WIDGET_PADDING - (widgetRect.width() - width));
+            } else {
+                iconRect.moveLeft(WIDGET_PADDING + (widgetRect.width() - width));
+            }
+        }
 
-        titleRect.moveLeft(WIDGET_PADDING + iconRect.right());
-        descriptionRect.moveLeft(WIDGET_PADDING + iconRect.right());
+        if (rtl) {
+            titleRect.moveRight(- WIDGET_PADDING + iconRect.left());
+            descriptionRect.moveRight(- WIDGET_PADDING + iconRect.left());
+        } else {
+            titleRect.moveLeft(WIDGET_PADDING + iconRect.right());
+            descriptionRect.moveLeft(WIDGET_PADDING + iconRect.right());
+        }
 
         if (!d->icon.isNull() || d->iconInSvg.isValid()) {  // using real painter...
             QRect rect(QPoint(lround(iconRect.left()), lround(iconRect.top())), d->iconSize);
@@ -276,8 +338,10 @@ void BasicWidget::paintForeground(QPainter * painter)
 
         if (!d->title.isEmpty()) {
             painter->setFont(titleFont);
-            painter->drawText(titleRect,
-                    Qt::AlignLeft | Qt::AlignTop | Qt::TextSingleLine, d->title);
+            // painter->drawText(titleRect,
+            //         Qt::AlignLeft | Qt::AlignTop | Qt::TextSingleLine, d->title);
+            drawText(painter, titleRect,
+                    Qt::AlignLeft | Qt::AlignTop | Qt::TextSingleLine, d->title, true);
         }
 
         if (!d->description.isEmpty()) {
@@ -288,17 +352,29 @@ void BasicWidget::paintForeground(QPainter * painter)
                 painter->setPen(QPen(clr));
             }
             painter->setFont(descriptionFont);
-            painter->drawText(descriptionRect,
-                    Qt::AlignLeft | Qt::AlignTop | Qt::TextSingleLine, d->description);
+            // painter->drawText(descriptionRect,
+            //         Qt::AlignLeft | Qt::AlignTop | Qt::TextSingleLine, d->description);
+            drawText(painter, descriptionRect,
+                    Qt::AlignLeft | Qt::AlignTop | Qt::TextSingleLine, d->description, false);
         }
     }
 
-    QLinearGradient gradient = QLinearGradient(
-            QPointF(size().width() - WIDGET_PADDING - 20, 0),
-            QPointF(size().width() - WIDGET_PADDING, 0)
-            );
-    gradient.setColorAt(1, Qt::transparent);
-    gradient.setColorAt(0, Qt::black);
+    QLinearGradient gradient;
+    if (QApplication::isRightToLeft()) {
+        gradient = QLinearGradient(
+                QPointF(WIDGET_PADDING, 0),
+                QPointF(WIDGET_PADDING + 20, 0)
+                );
+        gradient.setColorAt(0, Qt::transparent);
+        gradient.setColorAt(1, Qt::black);
+    } else {
+        gradient = QLinearGradient(
+                QPointF(size().width() - WIDGET_PADDING - 20, 0),
+                QPointF(size().width() - WIDGET_PADDING, 0)
+                );
+        gradient.setColorAt(1, Qt::transparent);
+        gradient.setColorAt(0, Qt::black);
+    }
     painter->setCompositionMode(QPainter::CompositionMode_DestinationIn);
     painter->fillRect(
             0, 0, (int)size().width(), (int)size().height(),
@@ -313,7 +389,9 @@ void BasicWidget::paintForeground(QPainter * painter)
 
 void BasicWidget::setIconSize(QSize size)
 {
-    d->iconSize = size; update();
+    d->iconSize = size;
+    update();
+    updateGeometry();
 }
 
 QSize BasicWidget::iconSize() const
@@ -323,7 +401,9 @@ QSize BasicWidget::iconSize() const
 
 void BasicWidget::setIcon(QIcon icon)
 {
-    d->icon = icon; update();
+    d->icon = icon;
+    update();
+    updateGeometry();
 }
 
 QIcon BasicWidget::icon() const
@@ -335,6 +415,7 @@ void BasicWidget::setIconInSvg(const Plasma::Svg & icon)
 {
     d->iconInSvg.setImagePath(icon.imagePath());
     update();
+    updateGeometry();
 }
 
 Plasma::Svg & BasicWidget::iconInSvg() const
@@ -342,10 +423,23 @@ Plasma::Svg & BasicWidget::iconInSvg() const
     return d->iconInSvg;
 }
 
-void BasicWidget::setTitle(const QString & title)
+void BasicWidget::setTitle(const QString & value)
 {
-    d->title = title;
+    d->title = value;
+
+    QString title(value);
+    int pos = d->shortcutPosition(title);
+    if (pos > -1) {
+        setShortcutKey(title.at(pos));
+    }
+
     update();
+    updateGeometry();
+}
+
+void BasicWidget::setShortcutKey(const QString & key)
+{
+  Q_UNUSED(key);
 }
 
 QString BasicWidget::title() const
@@ -357,6 +451,7 @@ void BasicWidget::setDescription(const QString & description)
 {
     d->description = description;
     update();
+    updateGeometry();
 }
 
 QString BasicWidget::description() const
@@ -367,6 +462,7 @@ QString BasicWidget::description() const
 void BasicWidget::setInnerOrientation(Qt::Orientation position) {
     d->innerOrientation = position;
     update();
+    updateGeometry();
 }
 
 Qt::Orientation BasicWidget::innerOrientation() const
@@ -376,7 +472,8 @@ Qt::Orientation BasicWidget::innerOrientation() const
 
 void BasicWidget::setAlignment(Qt::Alignment alignment)
 {
-    d->alignment = alignment; update();
+    d->alignment = alignment;
+    update();
 }
 
 Qt::Alignment BasicWidget::alignment() const
@@ -388,10 +485,6 @@ QSizeF BasicWidget::sizeHint(Qt::SizeHint which, const QSizeF & constraint) cons
 {
     QSizeF result = QSizeF();
 
-    if (!L_WIDGET_IS_INITIALIZED) {
-        return result;
-    }
-
     switch (which) {
         case Qt::MinimumSize:
             result = d->iconSize;
@@ -401,19 +494,137 @@ QSizeF BasicWidget::sizeHint(Qt::SizeHint which, const QSizeF & constraint) cons
             break;
         default:
             // Do we need a more precise sizeHint?
-            result = d->iconSize + QSizeF(2 * WIDGET_PADDING, 2 * WIDGET_PADDING);
+            // result = d->iconSize + QSizeF(2 * WIDGET_PADDING, 2 * WIDGET_PADDING);
+            result = d->iconSize;
+            QFontMetrics titleMetrics = QFontMetrics(font());
+            QFontMetrics desctiprionMetrics =
+                    QFontMetrics(KGlobalSettings::smallestReadableFont());
+            QSizeF textSize = QSizeF(
+                    qMax(
+                        titleMetrics.width(d->title),
+                        desctiprionMetrics.width(d->description)
+                    ),
+                    (titleMetrics.height()) +
+                    (d->description.isEmpty()?0:desctiprionMetrics.height())
+                );
+
             if (d->innerOrientation == Qt::Horizontal) {
-                result.scale(5.0, 1.0, Qt::IgnoreAspectRatio);
+                result.rwidth() += textSize.width();
+
+                if (result.height() < textSize.height()) {
+                    result.setHeight(textSize.height());
+                }
             } else {
-                result.scale(2.0, 2.0, Qt::IgnoreAspectRatio);
+                result.rheight() += textSize.height();
+
+                if (result.width() < textSize.width()) {
+                    result.setWidth(textSize.width());
+                }
             }
+            result += Widget::sizeHint(which, constraint) +
+                QSizeF(3 * WIDGET_PADDING, 2 * WIDGET_PADDING);
     }
-    if (constraint != QSizeF(-1, -1)) {
+
+    if (constraint.isValid()) {
         result = result.boundedTo(constraint);
     }
+
     return result;
 }
 
+void BasicWidget::drawText(QPainter * painter, const QRectF & rectangle, int flags, const QString & txt, bool shortcutEnabled)
+{
+    Q_UNUSED(flags);
+
+    if (txt.isEmpty()) {
+        return;
+    }
+
+    QString text = txt;
+    int shortcutPosition = d->shortcutPosition(text);
+    if (shortcutPosition > -1 && shortcutEnabled) {
+        text = text.remove(shortcutPosition - 1, 1);
+    } else {
+        shortcutEnabled = false;
+    }
+
+    static const int radius = 2;
+    if (group()->hasProperty("BlurTextShadow")) {
+        QColor textColor = painter->pen().color();
+        QColor shadowColor;
+        if (textColor.valueF() * textColor.alphaF() > 0.4) {
+            shadowColor = Qt::black;
+        } else {
+            shadowColor = Qt::white;
+        }
+
+        QPixmap result = Plasma::PaintUtils::shadowText(
+                text, textColor, shadowColor,
+                QPoint(0, 0), radius);
+
+        if (group()->hasProperty("TextColorBackground")) {
+            QColor bgColor;
+            if (!isEnabled()) {
+                bgColor = group()->backgroundColor()->disabled;
+            } else if (isHovered()) {
+                bgColor = group()->backgroundColor()->active;
+            } else {
+                bgColor = group()->backgroundColor()->normal;
+            }
+            painter->setRenderHint(QPainter::Antialiasing);
+            QRectF frect = QRectF(rectangle.topLeft(), result.size());
+            painter->fillPath(
+                    Plasma::PaintUtils::roundedRectangle(
+                        frect, 2 * radius), QBrush(bgColor)
+                    );
+        }
+
+        painter->drawPixmap(rectangle.topLeft(), result);
+
+        if (shortcutEnabled) {
+            int width = painter->boundingRect(
+                    rectangle,
+                    Qt::AlignLeft | Qt::AlignTop | Qt::TextSingleLine,
+                    text.left(shortcutPosition - 1)).width();
+            QPixmap result = Plasma::PaintUtils::shadowText(
+                    "_", textColor, shadowColor,
+                    QPoint(0, 0), radius);
+            painter->drawPixmap(rectangle.topLeft() + QPoint(width, 0), result);
+        }
+    } else {
+        if (group()->hasProperty("TextColorBackground")) {
+            QColor bgColor;
+            if (!isEnabled()) {
+                bgColor = group()->backgroundColor()->disabled;
+            } else if (isHovered()) {
+                bgColor = group()->backgroundColor()->active;
+            } else {
+                bgColor = group()->backgroundColor()->normal;
+            }
+            painter->setRenderHint(QPainter::Antialiasing);
+            QRectF frect = painter->boundingRect(rectangle,
+                    Qt::AlignLeft | Qt::AlignTop | Qt::TextSingleLine, text);
+            frect.adjust(- radius, - radius, radius, radius);
+            painter->fillPath(
+                    Plasma::PaintUtils::roundedRectangle(
+                        frect, 2*radius), QBrush(bgColor)
+                    );
+        }
+        painter->drawText(rectangle,
+                Qt::AlignLeft | Qt::AlignTop | Qt::TextSingleLine, text);
+
+        if (shortcutEnabled) {
+            int width = painter->boundingRect(
+                    rectangle,
+                    Qt::AlignLeft | Qt::AlignTop | Qt::TextSingleLine,
+                    text.left(shortcutPosition - 1)).width();
+            painter->drawText(
+                    QRectF(rectangle.topLeft() + QPoint(width, 0), rectangle.size()),
+                    QString('_'));
+        }
+    }
+}
+
 } // namespace Lancelot
 
 #include "BasicWidget.moc"
