@@ -37,7 +37,7 @@ class CachedProvider : public ComicProvider
          * @param identifier The identifier of the cached comic.
          * @param parent The parent object.
          */
-        CachedProvider( QObject *parent, const QVariantList &args = QVariantList() );
+        explicit CachedProvider( QObject *parent, const QVariantList &args = QVariantList() );
 
         /**
          * Destroys the cached provider.
@@ -110,6 +110,16 @@ class CachedProvider : public ComicProvider
          */
         virtual QString name() const;
 
+        /**
+        * Returns wether the comic is leftToRight or not
+        */
+        virtual bool isLeftToRight() const;
+
+        /**
+        * Returns wether the comic is topToBottom or not
+        */
+        virtual bool isTopToBottom() const;
+
         /**
          * Returns whether a comic with the given @p identifier is cached.
          */
@@ -130,6 +140,11 @@ class CachedProvider : public ComicProvider
          */
         virtual KUrl websiteUrl() const;
 
+        /**
+         * Returns the shop website of the comic.
+         */
+        virtual KUrl shopUrl() const;
+
     private Q_SLOTS:
         void triggerFinished();
 };
