@@ -18,11 +18,17 @@
 #include "converterrunner.h"
 #include <QApplication>
 #include <QClipboard>
+#include <QSet>
 #include <KIcon>
-#include <converter.h>
+#include <KDebug>
+#include <KToolInvocation>
+#include <KUnitConversion/Converter>
+#include <KUnitConversion/UnitCategory>
 
 #define CONVERSION_CHAR '>'
 
+using namespace KUnitConversion;
+
 class StringParser
 {
 public:
@@ -84,7 +90,7 @@ public:
     void pass(const QStringList &strings)
     {
         passWhiteSpace();
-        QString temp = m_s.mid(m_index);
+        const QString temp = m_s.mid(m_index);
 
         foreach (const QString& s, strings) {
             if (temp.startsWith(s)) {
@@ -120,10 +126,18 @@ ConverterRunner::ConverterRunner(QObject* parent, const QVariantList &args)
     Q_UNUSED(args)
     setObjectName("Converter");
 
+    m_separators << QString(CONVERSION_CHAR);
+    m_separators << i18nc("list of words that can used as amount of 'unit1' [in|to|as] 'unit2'",
+                          "in;to;as").split(';');
+
     //can not ignore commands: we have things like m4
     setIgnoredTypes(Plasma::RunnerContext::Directory | Plasma::RunnerContext::File |
                     Plasma::RunnerContext::NetworkLocation);
 
+    QString description = i18n("Converts the value of :q: when :q: is made up of "
+                               "\"value unit [>, to, as, in] unit\". You can use the "
+                               "Unit converter applet to find all available units.");
+    addSyntax(Plasma::RunnerSyntax("=:q:", description));
 }
 
 ConverterRunner::~ConverterRunner()
@@ -154,26 +168,53 @@ void ConverterRunner::match(Plasma::RunnerContext &context)
         }
     }
 
-    QStringList separators;
-    separators << QString(CONVERSION_CHAR) << i18nc("amount of <unit1> in <unit2>", "in")
-               << i18nc("amount of <unit1> as <unit2>", "as");
-    QString s = cmd.get(StringParser::GetString);
+    const QString s = cmd.get(StringParser::GetString);
 
-    if (!s.isEmpty() && !separators.contains(s)) {
+    if (!s.isEmpty() && !m_separators.contains(s)) {
         unit1 += ' ' + s;
     }
-    if (s.isEmpty() || !separators.contains(s)) {
-        cmd.pass(separators);
+    if (s.isEmpty() || !m_separators.contains(s)) {
+        cmd.pass(m_separators);
     }
     unit2 = cmd.rest();
 
-    Value v = Converter::self()->convert(Value(value, unit1), unit2);
+    Converter converter;
+    UnitCategory* category = converter.categoryForUnit(unit1);
+    if (!category) {
+        return;
+    }
+    Value v = category->convert(Value(value, unit1), unit2);
     if (v.isValid()) {
         Plasma::QueryMatch match(this);
-        match.setType(Plasma::QueryMatch::ExactMatch);
+        match.setType(Plasma::QueryMatch::InformationalMatch);
         match.setIcon(KIcon("edit-copy"));
         match.setText(v.toString());
-        match.setData(v.number);
+        match.setData(v.number());
+        context.addMatch(term, match);
+    } else if (!unit2.isEmpty()) {
+        const QStringList units = category->allUnits();
+        QSet<UnitPtr> matchingUnits;
+        foreach (const QString& s, units) {
+            if (s.startsWith(unit2, Qt::CaseInsensitive)) {
+                matchingUnits << category->unit(s);
+            }
+        }
+        foreach (const UnitPtr& u, matchingUnits) {
+            v = category->convert(Value(value, unit1), u->symbol());
+            Plasma::QueryMatch match(this);
+            match.setType(Plasma::QueryMatch::InformationalMatch);
+            match.setIcon(KIcon("edit-copy"));
+            match.setText(v.toString());
+            match.setData(v.number());
+            context.addMatch(term, match);
+        }
+    }
+    if (v.isValid() && !v.unit()->category()->description().isEmpty()) {
+        Plasma::QueryMatch match(this);
+        match.setType(Plasma::QueryMatch::PossibleMatch);
+        match.setIcon(KIcon("document-open-remote"));
+        match.setText(v.unit()->category()->description());
+        match.setData(v.unit()->category()->url().prettyUrl());
         context.addMatch(term, match);
     }
 }
@@ -181,7 +222,12 @@ void ConverterRunner::match(Plasma::RunnerContext &context)
 void ConverterRunner::run(const Plasma::RunnerContext &context, const Plasma::QueryMatch &match)
 {
     Q_UNUSED(context)
-    QApplication::clipboard()->setText(match.data().toString());
+    const QString data = match.data().toString();
+    if (data.startsWith(QLatin1String("http://"))) {
+        KToolInvocation::invokeBrowser(data);
+    } else {
+        QApplication::clipboard()->setText(data);
+    }
 }
 
 #include "converterrunner.moc"
