@@ -40,13 +40,14 @@ KateSessions::KateSessions(QObject *parent, const QVariantList& args)
 {
     Q_UNUSED(args);
     setObjectName("Kate Sessions");
+    setIgnoredTypes(Plasma::RunnerContext::FileSystem | Plasma::RunnerContext::NetworkLocation);
     m_icon = KIcon("kate");
 
     loadSessions();
 
     // listen for changes to the list of kate sessions
     KDirWatch *historyWatch = new KDirWatch(this);
-    QStringList sessiondirs = KGlobal::dirs()->findDirs("data", "kate/sessions/");
+    const QStringList sessiondirs = KGlobal::dirs()->findDirs("data", "kate/sessions/");
     foreach (const QString &dir, sessiondirs) {
         historyWatch->addDir(dir);
     }
@@ -54,6 +55,11 @@ KateSessions::KateSessions(QObject *parent, const QVariantList& args)
     connect(historyWatch,SIGNAL(created(QString)),this,SLOT(loadSessions()));
     connect(historyWatch,SIGNAL(deleted(QString)),this,SLOT(loadSessions()));
 
+    Plasma::RunnerSyntax s(":q:", i18n("Finds Kate sessions matching :q:."));
+    s.addExampleQuery("kate :q:");
+    addSyntax(s);
+
+    addSyntax(Plasma::RunnerSyntax("kate", i18n("Lists all the Kate editor sessions in your account.")));
 }
 
 KateSessions::~KateSessions()
@@ -95,11 +101,11 @@ void KateSessions::match(Plasma::RunnerContext &context)
 
     bool list_all = false;
 
-    if (term.toLower().startsWith("kate")) {
+    if (term.toLower().startsWith(QLatin1String("kate"))) {
         if (term.toLower().trimmed() == "kate") {
             list_all = true;
         }
-        term.replace("kate", "", Qt::CaseInsensitive);
+        term.remove("kate", Qt::CaseInsensitive);
         term = term.trimmed();
     }
 
@@ -108,6 +114,10 @@ void KateSessions::match(Plasma::RunnerContext &context)
     }
 
     foreach (const QString &session, m_sessions) {
+        if (!context.isValid()) {
+            return;
+        }
+
         if (list_all || (!term.isEmpty() && session.contains(term, Qt::CaseInsensitive))) {
             Plasma::QueryMatch match(this);
             match.setType(Plasma::QueryMatch::PossibleMatch);
@@ -141,7 +151,7 @@ void KateSessions::run(const Plasma::RunnerContext &context, const Plasma::Query
 
     if (!session.isEmpty()) {
         QStringList args;
-        args << "--start" << session;
+       	args << "--start" << session;
         KToolInvocation::kdeinitExec("kate", args);
     }
 }
